#ifndef elec_det_h__
#define elec_det_h__


#include "stdint.h"
#include "bytearray.h"
#include "interface/basechecker.h"

int elec_init(void);

int elec_scheme_init(void);

int elec_scheme_deinit(void);

uint8_t elec_local_addr(void);

int elec_hard_version(void);

void elec_led1_power(int power);

void elec_led2_power(int power);

array_def *elec_bootinfo(void);

array_def *elec_check_result(void);

void elec_check_result_clear(void);

array_def *elec_write_hardversion(int version);

array_def *elec_write_resistor_cbv(int cbv);

array_def *elec_write_fat_buff(array_def *d);

array_def *elec_check_resistor(uint8_t index);

array_def *elec_check_with_scheme(array_def *uid_psw);

array_def *elec_code(array_def *uid_psw);

array_def *elec_check_slot(void);

int elec_check_load_task_param(uint8_t task_index);

void elec_exe_task(void);

int elec_task_env_back(void);

int elec_task_env_restore(void);

CheckerTask elec_get_task_fun(uint8_t taskid);


#endif



